#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/param.h>
#include <unistd.h>

#include "register_ioctl.h"

enum state parse_state(char *state_in) {
	static const char state_off[] = "off";
	static const char state_on[]  = "on";
	static size_t state_off_len = sizeof(state_off) - 1;
	static size_t state_on_len  = sizeof(state_on) - 1;

	size_t state_in_len  = strnlen(state_in, 16);

	if(strncmp(state_off, state_in, MAX(state_off_len, state_in_len)) == 0) {
		return STATE_OFF;
	} else if(strncmp(state_on, state_in, MAX(state_on_len, state_in_len)) == 0) {
		return STATE_ON;
	}

	return STATE_UNKNOWN;
}

enum command parse_command(char *command_in) {
	static const char command_wp[]   = "wp";
	static const char command_smap[] = "smap";
	static const char command_smep[] = "smep";
	static size_t command_wp_len   = sizeof(command_wp) - 1;
	static size_t command_smap_len = sizeof(command_smap) - 1;
	static size_t command_smep_len = sizeof(command_smep) - 1;

	size_t command_in_len   = strnlen(command_in, 16);

	if(strncmp(command_wp, command_in, MAX(command_wp_len, command_in_len)) == 0) {
		return COMMAND_WP;
	} else if(strncmp(command_smap, command_in, MAX(command_smap_len, command_in_len)) == 0) {
		return COMMAND_SMAP;
	} else if(strncmp(command_smep, command_in, MAX(command_smep_len, command_in_len)) == 0) {
		return COMMAND_SMEP;
	}

	return COMMAND_UNKNOWN;
}

int main(int argc, char **argv) {
	if(argc < 3) {
		printf("Error: Not enough arguments!\n");
		return -1;
	}

	enum command command = parse_command(argv[1]);
	if(command == COMMAND_UNKNOWN) {
		printf("Error: Command not recognized (%s)!\n", argv[1]);
		return -2;
	}

	enum state state = parse_state(argv[2]);
	if(state == STATE_UNKNOWN) {
		printf("Error: State not recognized (%s)!\n", argv[2]);
		return -3;
	}

	int dev_fd = open("/dev/control_registers", O_RDWR);
	if(dev_fd == -1) {
		printf("Error opening device file: %s\n", strerror(errno));
		return -4;
	}

	if(ioctl(dev_fd, command, state) == -1) {
		close(dev_fd);
		printf("Error sending ioctl to device file: %s\n", strerror(errno));
		return -5;
	}

	printf("Sending ioctl was successful!\n");
	close(dev_fd);

	return 0;
}
